@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,45 +33,60 @@ class CControllerProfileUpdate extends CController {
 		if ($ret) {
 			switch ($this->getInput('idx')) {
 				case 'web.actionconf.filter.active':
+				case 'web.applications.filter.active':
 				case 'web.auditacts.filter.active':
-				case 'web.auditlogs.filter.active':
+				case 'web.auditlog.filter.active':
 				case 'web.avail_report.filter.active':
-				case 'web.graphs.filter.active':
+				case 'web.charts.filter.active':
 				case 'web.correlation.filter.active':
+				case 'web.dashbrd.filter.active':
+				case 'web.discovery.filter.active':
 				case 'web.discoveryconf.filter.active':
+				case 'web.graphs.filter.active':
 				case 'web.groups.filter.active':
+				case 'web.host_discovery.filter.active':
 				case 'web.hostinventories.filter.active':
+				case 'web.hostinventoriesoverview.filter.active':
+				case 'web.hosts.filter.active':
+				case 'web.hostsmon.filter.active':
 				case 'web.httpconf.filter.active':
 				case 'web.httpdetails.filter.active':
-				case 'web.hosts.filter.active':
-				case 'web.items.filter.active':
 				case 'web.item.graph.filter.active':
+				case 'web.items.filter.active':
 				case 'web.latest.filter.active':
 				case 'web.layout.mode':
 				case 'web.maintenance.filter.active':
 				case 'web.media_types.filter.active':
+				case 'web.modules.filter.active':
 				case 'web.overview.filter.active':
 				case 'web.problem.filter.active':
 				case 'web.proxies.filter.active':
+				case 'web.screenconf.filter.active':
+				case 'web.screens.filter.active':
 				case 'web.scripts.filter.active':
+				case 'web.search.hats.'.WIDGET_SEARCH_HOSTS.'.state':
+				case 'web.search.hats.'.WIDGET_SEARCH_TEMPLATES.'.state':
+				case 'web.search.hats.'.WIDGET_SEARCH_HOSTGROUP.'.state':
+				case 'web.sidebar.mode':
+				case 'web.slideconf.filter.active':
+				case 'web.slides.filter.active':
+				case 'web.sysmapconf.filter.active':
 				case 'web.templates.filter.active':
 				case 'web.toptriggers.filter.active':
+				case 'web.tr_events.hats.'.WIDGET_HAT_EVENTACTIONS.'.state':
+				case 'web.tr_events.hats.'.WIDGET_HAT_EVENTLIST.'.state':
 				case 'web.triggers.filter.active':
-				case 'web.screens.filter.active':
-				case 'web.screenconf.filter.active':
-				case 'web.slides.filter.active':
-				case 'web.slideconf.filter.active':
-				case 'web.sysmapconf.filter.active':
 				case 'web.user.filter.active':
 				case 'web.usergroup.filter.active':
-				case 'web.dashbrd.filter.active':
+				case 'web.web.filter.active':
+				case CControllerDashboardView::DYNAMIC_ITEM_HOST_PROFILE_KEY:
 					$ret = true;
 					break;
 
+				case !!preg_match('/web.dashbrd.navtree-\d+.toggle/', $this->getInput('idx')):
+				case 'web.dashbrd.navtree.item.selected':
 				case 'web.latest.toggle':
 				case 'web.latest.toggle_other':
-				case 'web.dashbrd.navtree.item.selected':
-				case !!preg_match('/web.dashbrd.navtree-\d+.toggle/', $this->getInput('idx')):
 					$ret = $this->hasInput('idx2');
 					break;
 
@@ -117,7 +132,11 @@ class CControllerProfileUpdate extends CController {
 				break;
 
 			case 'web.layout.mode':
-				CView::setLayoutMode($value_int);
+				CViewHelper::saveLayoutMode($value_int);
+				break;
+
+			case 'web.sidebar.mode':
+				CViewHelper::saveSidebarMode($value_int);
 				break;
 
 			default:
