@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,8 +19,13 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $this->addJsFile('multiselect.js');
-$this->includeJSfile('app/views/administration.script.edit.js.php');
+
+$this->includeJsFile('administration.script.edit.js.php');
 
 $widget = (new CWidget())->setTitle(_('Scripts'));
 
@@ -115,7 +120,10 @@ $scriptFormList->addRow(new CLabel(_('Confirmation text'), 'confirmation'), [
 $scriptView = (new CTabView())->addTab('scripts', _('Script'), $scriptFormList);
 
 // footer
-$cancelButton = (new CRedirectButton(_('Cancel'), 'zabbix.php?action=script.list'))->setId('cancel');
+$cancelButton = (new CRedirectButton(_('Cancel'), (new CUrl('zabbix.php'))
+	->setArgument('action', 'script.list')
+	->setArgument('page', CPagerHelper::loadPage('script.list', null))
+))->setId('cancel');
 
 if ($data['scriptid'] == 0) {
 	$addButton = (new CSubmitButton(_('Add'), 'action', 'script.create'))->setId('add');
