@@ -25,7 +25,9 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 $page['title'] = _('Configuration of hosts');
 $page['file'] = 'hosts.php';
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
-$page['scripts'] = ['multiselect.js', 'textareaflexible.js'];
+$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'class.cviewswitcher.js', 'class.cverticalaccordion.js',
+	'inputsecret.js', 'macrovalue.js'
+];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -40,7 +42,6 @@ $fields = [
 	'hostids' =>				[T_ZBX_INT, O_OPT, P_SYS,			DB_ID,		null],
 	'groupids' =>				[T_ZBX_INT, O_OPT, P_SYS,			DB_ID,		null],
 	'applications' =>			[T_ZBX_INT, O_OPT, P_SYS,			DB_ID,		null],
-	'groupid' =>				[T_ZBX_INT, O_OPT, P_SYS,			DB_ID,		null],
 	'hostid' =>					[T_ZBX_INT, O_OPT, P_SYS,			DB_ID,		'isset({form}) && {form} == "update"'],
 	'clone_hostid' =>			[T_ZBX_INT, O_OPT, P_SYS,			DB_ID,
 									'isset({form}) && {form} == "full_clone"'
@@ -55,7 +56,7 @@ $fields = [
 									IN([HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED]), null
 								],
 	'interfaces' =>				[T_ZBX_STR, O_OPT, null,			NOT_EMPTY,
-									'isset({add}) || isset({update})', _('Agent or SNMP or JMX or IPMI interface')
+									'isset({add}) || isset({update})'
 								],
 	'mainInterfaces' =>			[T_ZBX_INT, O_OPT, null,			DB_ID,		null],
 	'tags' =>					[T_ZBX_STR, O_OPT, null,			null,		null],
@@ -63,6 +64,14 @@ $fields = [
 									IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE]),
 									null
 								],
+	'mass_update_macros' => 	[T_ZBX_INT, O_OPT, null,
+									IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE, ZBX_ACTION_REMOVE_ALL]),
+									null
+								],
+	'macros_add' =>				[T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'macros_update' =>			[T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'macros_remove' =>			[T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'macros_remove_all' =>		[T_ZBX_INT, O_OPT, null, IN([0,1]), null],
 	'templates' =>				[T_ZBX_INT, O_OPT, null,			DB_ID,		null],
 	'add_templates' =>			[T_ZBX_INT, O_OPT, null,			DB_ID,		null],
 	'templates_rem' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT,		null,		null],
@@ -88,8 +97,11 @@ $fields = [
 	'flags' =>					[T_ZBX_INT, O_OPT, null,
 									IN([ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CREATED]), null
 								],
-	'mass_replace_tpls' =>		[T_ZBX_STR, O_OPT, null,			null,		null],
 	'mass_clear_tpls' =>		[T_ZBX_STR, O_OPT, null,			null,		null],
+	'mass_action_tpls' =>		[T_ZBX_INT, O_OPT, null,
+									IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE]),
+									null
+								],
 	'inventory_mode' =>			[T_ZBX_INT, O_OPT, null,
 									IN(HOST_INVENTORY_DISABLED.','.HOST_INVENTORY_MANUAL.','.HOST_INVENTORY_AUTOMATIC),
 									null
@@ -123,6 +135,7 @@ $fields = [
 	'filter_rst' =>				[T_ZBX_STR, O_OPT, P_SYS,			null,		null],
 	'filter_host' =>			[T_ZBX_STR, O_OPT, null,			null,		null],
 	'filter_templates' =>		[T_ZBX_INT, O_OPT, null,			DB_ID,		null],
+	'filter_groups' =>			[T_ZBX_INT, O_OPT, null,			DB_ID,		null],
 	'filter_ip' =>				[T_ZBX_STR, O_OPT, null,			null,		null],
 	'filter_dns' =>				[T_ZBX_STR, O_OPT, null,			null,		null],
 	'filter_port' =>			[T_ZBX_STR, O_OPT, null,			null,		null],
@@ -145,9 +158,6 @@ check_fields($fields);
 /*
  * Permissions
  */
-if (getRequest('groupid') && !isWritableHostGroups([getRequest('groupid')])) {
-	access_deny();
-}
 if (getRequest('hostid')) {
 	$hosts = API::Host()->get([
 		'output' => [],
@@ -172,6 +182,7 @@ if (hasRequest('filter_set')) {
 		PROFILE_TYPE_INT
 	);
 	CProfile::updateArray('web.hosts.filter_templates', getRequest('filter_templates', []), PROFILE_TYPE_ID);
+	CProfile::updateArray('web.hosts.filter_groups', getRequest('filter_groups', []), PROFILE_TYPE_ID);
 	CProfile::updateArray('web.hosts.filter_proxyids', getRequest('filter_proxyids', []), PROFILE_TYPE_ID);
 	CProfile::update('web.hosts.filter.evaltype', getRequest('filter_evaltype', TAG_EVAL_TYPE_AND_OR),
 		PROFILE_TYPE_INT
@@ -199,6 +210,7 @@ elseif (hasRequest('filter_rst')) {
 	CProfile::delete('web.hosts.filter_port');
 	CProfile::delete('web.hosts.filter_monitored_by');
 	CProfile::deleteIdx('web.hosts.filter_templates');
+	CProfile::deleteIdx('web.hosts.filter_groups');
 	CProfile::deleteIdx('web.hosts.filter_proxyids');
 	CProfile::delete('web.hosts.filter.evaltype');
 	CProfile::deleteIdx('web.hosts.filter.tags.tag');
@@ -212,6 +224,7 @@ $filter = [
 	'dns' => CProfile::get('web.hosts.filter_dns', ''),
 	'host' => CProfile::get('web.hosts.filter_host', ''),
 	'templates' => CProfile::getArray('web.hosts.filter_templates', []),
+	'groups' => CProfile::getArray('web.hosts.filter_groups', []),
 	'port' => CProfile::get('web.hosts.filter_port', ''),
 	'monitored_by' => CProfile::get('web.hosts.filter_monitored_by', ZBX_MONITORED_BY_ANY),
 	'proxyids' => CProfile::getArray('web.hosts.filter_proxyids', []),
@@ -226,6 +239,7 @@ foreach (CProfile::getArray('web.hosts.filter.tags.tag', []) as $i => $tag) {
 		'operator' => CProfile::get('web.hosts.filter.tags.operator', null, $i)
 	];
 }
+CArrayHelper::sort($filter['tags'], ['tag', 'value', 'operator']);
 
 $tags = getRequest('tags', []);
 foreach ($tags as $key => $tag) {
@@ -248,12 +262,11 @@ foreach ($tags as $key => $tag) {
 $macros = cleanInheritedMacros(getRequest('macros', []));
 
 // Remove empty new macro lines.
-foreach ($macros as $idx => $macro) {
-	if (!array_key_exists('hostmacroid', $macro) && $macro['macro'] === '' && $macro['value'] === ''
-			&& $macro['description'] === '') {
-		unset($macros[$idx]);
-	}
-}
+$macros = array_filter($macros, function($macro) {
+	$keys = array_flip(['hostmacroid', 'macro', 'value', 'description']);
+
+	return (bool) array_filter(array_intersect_key($macro, $keys));
+});
 
 /*
  * Actions
@@ -325,6 +338,23 @@ elseif (hasRequest('hostid') && (hasRequest('clone') || hasRequest('full_clone')
 		$_REQUEST['clone_hostid'] = $_REQUEST['hostid'];
 	}
 
+	if ($macros && in_array(ZBX_MACRO_TYPE_SECRET, array_column($macros, 'type'))) {
+		// Reset macro type and value.
+		$macros = array_map(function($value) {
+			return ($value['type'] == ZBX_MACRO_TYPE_SECRET)
+				? ['value' => '', 'type' => ZBX_MACRO_TYPE_TEXT] + $value
+				: $value;
+		}, $macros);
+
+		$msg = [
+			'type' => 'error',
+			'message' => _('The cloned host contains user defined macros with type "Secret text". The value and type of these macros were reset.'),
+			'src' => ''
+		];
+
+		echo makeMessageBox(false, [$msg], null, true, false)->addClass(ZBX_STYLE_MSG_WARNING);
+	}
+
 	unset($_REQUEST['hostid'], $_REQUEST['flags']);
 }
 elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && hasRequest('masssave')) {
@@ -347,7 +377,8 @@ elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && ha
 			$options['selectGroups'] = ['groupid'];
 		}
 
-		if (array_key_exists('templates', $visible) && !hasRequest('mass_replace_tpls')) {
+		if (array_key_exists('templates', $visible)
+				&& !(getRequest('mass_action_tpls') == ZBX_ACTION_REPLACE && !hasRequest('mass_clear_tpls'))) {
 			$options['selectParentTemplates'] = ['templateid'];
 		}
 
@@ -367,6 +398,15 @@ elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && ha
 			$tags = array_values($unique_tags);
 		}
 
+		if (array_key_exists('macros', $visible)) {
+			$mass_update_macros = getRequest('mass_update_macros', ZBX_ACTION_ADD);
+
+			if ($mass_update_macros == ZBX_ACTION_ADD || $mass_update_macros == ZBX_ACTION_REPLACE
+					|| $mass_update_macros == ZBX_ACTION_REMOVE) {
+				$options['selectMacros'] = ['hostmacroid', 'macro'];
+			}
+		}
+
 		$hosts = API::Host()->get($options);
 
 		if (array_key_exists('groups', $visible)) {
@@ -419,27 +459,6 @@ elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && ha
 			$new_values['status'] = getRequest('status', HOST_STATUS_NOT_MONITORED);
 		}
 
-		$templateids = [];
-		if (array_key_exists('templates', $visible)) {
-			$templateids = $_REQUEST['templates'];
-
-			if (hasRequest('mass_replace_tpls')) {
-				if (hasRequest('mass_clear_tpls')) {
-					$host_templates = API::Template()->get([
-						'output' => ['templateid'],
-						'hostids' => $hostids
-					]);
-
-					$host_templateids = zbx_objectValues($host_templates, 'templateid');
-					$templates_to_delete = array_diff($host_templateids, $templateids);
-
-					$new_values['templates_clear'] = zbx_toObject($templates_to_delete, 'templateid');
-				}
-
-				$new_values['templates'] = $templateids;
-			}
-		}
-
 		$host_inventory = array_intersect_key(getRequest('host_inventory', []), $visible);
 
 		if (array_key_exists('inventory_mode', $visible)) {
@@ -467,6 +486,9 @@ elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && ha
 			}
 		}
 
+		$host_macros_add = [];
+		$host_macros_update = [];
+		$host_macros_remove = [];
 		foreach ($hosts as &$host) {
 			if (array_key_exists('groups', $visible)) {
 				if ($new_groupids && $mass_update_groups == ZBX_ACTION_ADD) {
@@ -484,10 +506,32 @@ elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && ha
 				}
 			}
 
-			if ($templateids && array_key_exists('parentTemplates', $host)) {
-				$host['templates'] = array_unique(
-					array_merge($templateids, zbx_objectValues($host['parentTemplates'], 'templateid'))
-				);
+			if (array_key_exists('templates', $visible)) {
+				$host_templateids = array_key_exists('parentTemplates', $host)
+					? zbx_objectValues($host['parentTemplates'], 'templateid')
+					: [];
+
+				switch (getRequest('mass_action_tpls')) {
+					case ZBX_ACTION_ADD:
+						$host['templates'] = array_unique(array_merge($host_templateids, getRequest('templates', [])));
+						break;
+
+					case ZBX_ACTION_REPLACE:
+						$host['templates'] = getRequest('templates', []);
+						if (getRequest('mass_clear_tpls')) {
+							$host['templates_clear'] = array_unique(
+								array_diff($host_templateids, getRequest('templates', []))
+							);
+						}
+						break;
+
+					case ZBX_ACTION_REMOVE:
+						$host['templates'] = array_unique(array_diff($host_templateids, getRequest('templates', [])));
+						if (getRequest('mass_clear_tpls')) {
+							$host['templates_clear'] = array_unique(getRequest('templates', []));
+						}
+						break;
+				}
 			}
 
 			if (array_key_exists('inventory_mode', $new_values)) {
@@ -530,6 +574,81 @@ elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && ha
 				}
 			}
 
+			if (array_key_exists('macros', $visible)) {
+				switch ($mass_update_macros) {
+					case ZBX_ACTION_ADD:
+						if ($macros) {
+							$update_existing = getRequest('macros_add', 0);
+
+							foreach ($macros as $macro) {
+								foreach ($host['macros'] as $host_macro) {
+									if ($macro['macro'] === $host_macro['macro']) {
+										if ($update_existing) {
+											$macro['hostmacroid'] = $host_macro['hostmacroid'];
+											$host_macros_update[] = $macro;
+										}
+
+										continue 2;
+									}
+								}
+
+								$macro['hostid'] = $host['hostid'];
+								$host_macros_add[] = $macro;
+							}
+						}
+						break;
+
+					case ZBX_ACTION_REPLACE: // In Macros its update.
+						if ($macros) {
+							$add_missing = getRequest('macros_update', 0);
+
+							foreach ($macros as $macro) {
+								foreach ($host['macros'] as $host_macro) {
+									if ($macro['macro'] === $host_macro['macro']) {
+										$macro['hostmacroid'] = $host_macro['hostmacroid'];
+										$host_macros_update[] = $macro;
+
+										continue 2;
+									}
+								}
+
+								if ($add_missing) {
+									$macro['hostid'] = $host['hostid'];
+									$host_macros_add[] = $macro;
+								}
+							}
+						}
+						break;
+
+					case ZBX_ACTION_REMOVE:
+						if ($macros) {
+							$except_selected = getRequest('macros_remove', 0);
+
+							$macro_names = array_column($macros, 'macro');
+
+							foreach ($host['macros'] as $host_macro) {
+								if ((!$except_selected && in_array($host_macro['macro'], $macro_names))
+										|| ($except_selected && !in_array($host_macro['macro'], $macro_names))) {
+									$host_macros_remove[] = $host_macro['hostmacroid'];
+								}
+							}
+						}
+						break;
+
+					case ZBX_ACTION_REMOVE_ALL:
+						if (!getRequest('macros_remove_all', 0)) {
+							throw new Exception();
+						}
+
+						$host['macros'] = [];
+						break;
+				}
+
+				if ($mass_update_macros != ZBX_ACTION_REMOVE_ALL) {
+					unset($host['macros']);
+				}
+			}
+
 			unset($host['parentTemplates']);
 
 			$host = $new_values + $host;
@@ -540,6 +659,28 @@ elseif (hasRequest('action') && getRequest('action') === 'host.massupdate' && ha
 			throw new Exception();
 		}
 
+		/**
+		 * Macros must be updated separately, since calling API::UserMacro->replaceMacros() inside
+		 * API::Host->update() results in loss of secret macro values.
+		 */
+		if ($host_macros_remove) {
+			if (!API::UserMacro()->delete($host_macros_remove)) {
+				throw new Exception();
+			}
+		}
+
+		if ($host_macros_add) {
+			if (!API::UserMacro()->create($host_macros_add)) {
+				throw new Exception();
+			}
+		}
+
+		if ($host_macros_update) {
+			if (!API::UserMacro()->update($host_macros_update)) {
+				throw new Exception();
+			}
+		}
+
 		DBend(true);
 
 		uncheckTableRows();
@@ -609,11 +750,16 @@ elseif (hasRequest('add') || hasRequest('update')) {
 					continue;
 				}
 
-				if ($interface['type'] == INTERFACE_TYPE_SNMP && !isset($interface['bulk'])) {
-					$interfaces[$key]['bulk'] = SNMP_BULK_DISABLED;
-				}
-				else {
-					$interfaces[$key]['bulk'] = SNMP_BULK_ENABLED;
+				// Proccess SNMP interface fields.
+				if ($interface['type'] == INTERFACE_TYPE_SNMP) {
+					if (!array_key_exists('details', $interface)) {
+						unset($interface[$key]);
+						continue;
+					}
+
+					$interfaces[$key]['details']['bulk'] = array_key_exists('bulk', $interface['details'])
+						? SNMP_BULK_ENABLED
+						: SNMP_BULK_DISABLED;
 				}
 
 				if ($interface['isNew']) {
@@ -883,27 +1029,26 @@ elseif (hasRequest('hosts') && hasRequest('action') && str_in_array(getRequest('
 /*
  * Display
  */
-$pageFilter = new CPageFilter([
-	'groups' => [
-		'real_hosts' => true,
-		'editable' => true
-	],
-	'groupid' => getRequest('groupid')
-]);
-
-$_REQUEST['groupid'] = $pageFilter->groupid;
-$_REQUEST['hostid'] = getRequest('hostid', 0);
-
 $config = select_config();
 
 if (hasRequest('hosts') && (getRequest('action') === 'host.massupdateform' || hasRequest('masssave'))) {
 	$data = [
 		'hosts' => getRequest('hosts'),
 		'visible' => getRequest('visible', []),
-		'mass_replace_tpls' => getRequest('mass_replace_tpls'),
+		'mass_action_tpls' => getRequest('mass_action_tpls'),
 		'mass_clear_tpls' => getRequest('mass_clear_tpls'),
 		'groups' => getRequest('groups', []),
+		'mass_update_groups' => getRequest('mass_update_groups', ZBX_ACTION_ADD),
 		'tags' => $tags,
+		'mass_update_tags' => getRequest('mass_update_tags', ZBX_ACTION_ADD),
+		'macros' => $macros,
+		'macros_checkbox' => [
+			ZBX_ACTION_ADD => getRequest('macros_add', '0'),
+			ZBX_ACTION_REPLACE => getRequest('macros_update', '0'),
+			ZBX_ACTION_REMOVE => getRequest('macros_remove', '0'),
+			ZBX_ACTION_REMOVE_ALL => getRequest('macros_remove_all', '0')
+		],
+		'macros_visible' => getRequest('mass_update_macros', ZBX_ACTION_ADD),
 		'status' => getRequest('status', HOST_STATUS_MONITORED),
 		'description' => getRequest('description'),
 		'proxy_hostid' => getRequest('proxy_hostid', ''),
@@ -946,6 +1091,10 @@ if (hasRequest('hosts') && (getRequest('action') === 'host.massupdateform' || ha
 		]), ['templateid' => 'id'])
 		: [];
 
+	if (!$data['macros']) {
+		$data['macros'] = [['macro' => '', 'type' => ZBX_MACRO_TYPE_TEXT, 'value' => '', 'description' => '']];
+	}
+
 	$hostView = new CView('configuration.host.massupdate', $data);
 }
 elseif (hasRequest('form')) {
@@ -1000,8 +1149,6 @@ elseif (hasRequest('form')) {
 		'tls_psk' => getRequest('tls_psk', '')
 	];
 
-	$groups = [];
-
 	if (!hasRequest('form_refresh')) {
 		if ($data['hostid'] != 0) {
 			$dbHosts = API::Host()->get([
@@ -1011,7 +1158,7 @@ elseif (hasRequest('form')) {
 				],
 				'selectGroups' => ['groupid'],
 				'selectParentTemplates' => ['templateid'],
-				'selectMacros' => ['hostmacroid', 'macro', 'value', 'description'],
+				'selectMacros' => ['hostmacroid', 'macro', 'value', 'description', 'type'],
 				'selectDiscoveryRule' => ['itemid', 'name'],
 				'selectHostDiscovery' => ['parent_hostid'],
 				'selectInventory' => API_OUTPUT_EXTEND,
@@ -1030,7 +1177,7 @@ elseif (hasRequest('form')) {
 			$data['host'] = $dbHost['host'];
 			$data['visiblename'] = $dbHost['name'];
 			$data['interfaces'] = API::HostInterface()->get([
-				'output' => ['interfaceid', 'main', 'type', 'useip', 'ip', 'dns', 'port', 'bulk'],
+				'output' => API_OUTPUT_EXTEND,
 				'selectItems' => ['type'],
 				'hostids' => [$data['hostid']],
 				'sortfield' => 'interfaceid'
@@ -1096,9 +1243,7 @@ elseif (hasRequest('form')) {
 			$groups = zbx_objectValues($dbHost['groups'], 'groupid');
 		}
 		else {
-			if (getRequest('groupid', 0) != 0) {
-				$groups[] = getRequest('groupid');
-			}
+			$groups = getRequest('groupids', []);
 
 			$data['status'] = HOST_STATUS_MONITORED;
 		}
@@ -1192,10 +1337,12 @@ elseif (hasRequest('form')) {
 	$data['macros'] = array_values(order_macros($data['macros'], 'macro'));
 
 	if (!$data['macros'] && !$data['readonly']) {
-		$macro = ['macro' => '', 'value' => '', 'description' => ''];
+		$macro = ['macro' => '', 'value' => '', 'description' => '', 'type' => ZBX_MACRO_TYPE_TEXT];
+
 		if ($data['show_inherited_macros']) {
-			$macro['type'] = ZBX_PROPERTY_OWN;
+			$macro['inherited_type'] = ZBX_PROPERTY_OWN;
 		}
+
 		$data['macros'][] = $macro;
 	}
 
@@ -1294,6 +1441,23 @@ else {
 	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
 	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
+	// Get host groups.
+	$filter['groups'] = $filter['groups']
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $filter['groups'],
+			'real_hosts' => true,
+			'editable' => true,
+			'preservekeys' => true
+		]), ['groupid' => 'id'])
+		: [];
+
+	$filter_groupids = $filter['groups'] ? array_keys($filter['groups']) : null;
+	if ($filter_groupids) {
+		$filter_groupids = getSubGroups($filter_groupids);
+	}
+
+	// Get templates.
 	$filter['templates'] = $filter['templates']
 		? CArrayHelper::renameObjectsKeys(API::Template()->get([
 			'output' => ['templateid', 'name'],
@@ -1302,54 +1466,62 @@ else {
 		]), ['templateid' => 'id'])
 		: [];
 
-	// get Hosts
-	$hosts = [];
-	if ($pageFilter->groupsSelected) {
-		switch ($filter['monitored_by']) {
-			case ZBX_MONITORED_BY_ANY:
-				$proxyids = null;
-				break;
+	switch ($filter['monitored_by']) {
+		case ZBX_MONITORED_BY_ANY:
+			$proxyids = null;
+			break;
 
-			case ZBX_MONITORED_BY_PROXY:
-				$proxyids = $filter['proxyids']
-					? $filter['proxyids']
-					: array_keys(API::Proxy()->get([
-						'output' => [],
-						'preservekeys' => true
-					]));
-				break;
+		case ZBX_MONITORED_BY_PROXY:
+			$proxyids = $filter['proxyids']
+				? $filter['proxyids']
+				: array_keys(API::Proxy()->get([
+					'output' => [],
+					'preservekeys' => true
+				]));
+			break;
 
-			case ZBX_MONITORED_BY_SERVER:
-				$proxyids = 0;
-				break;
-		}
-
-		$hosts = API::Host()->get([
-			'output' => ['hostid', $sortField],
-			'evaltype' => $filter['evaltype'],
-			'tags' => $filter['tags'],
-			'groupids' => $pageFilter->groupids,
-			'templateids' => $filter['templates'] ? array_keys($filter['templates']) : null,
-			'editable' => true,
-			'sortfield' => $sortField,
-			'limit' => $config['search_limit'] + 1,
-			'search' => [
-				'name' => ($filter['host'] === '') ? null : $filter['host'],
-				'ip' => ($filter['ip'] === '') ? null : $filter['ip'],
-				'dns' => ($filter['dns'] === '') ? null : $filter['dns']
-			],
-			'filter' => [
-				'port' => ($filter['port'] === '') ? null : $filter['port']
-			],
-			'proxyids' => $proxyids
-		]);
+		case ZBX_MONITORED_BY_SERVER:
+			$proxyids = 0;
+			break;
 	}
+
+	// Select hosts.
+	$hosts = API::Host()->get([
+		'output' => ['hostid', $sortField],
+		'evaltype' => $filter['evaltype'],
+		'tags' => $filter['tags'],
+		'groupids' => $filter_groupids,
+		'templateids' => $filter['templates'] ? array_keys($filter['templates']) : null,
+		'editable' => true,
+		'sortfield' => $sortField,
+		'limit' => $config['search_limit'] + 1,
+		'search' => [
+			'name' => ($filter['host'] === '') ? null : $filter['host'],
+			'ip' => ($filter['ip'] === '') ? null : $filter['ip'],
+			'dns' => ($filter['dns'] === '') ? null : $filter['dns']
+		],
+		'filter' => [
+			'port' => ($filter['port'] === '') ? null : $filter['port']
+		],
+		'proxyids' => $proxyids
+	]);
+
 	order_result($hosts, $sortField, $sortOrder);
 
-	$url = (new CUrl('hosts.php'))
-		->setArgument('groupid', $pageFilter->groupid);
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$pagingLine = getPagingLine($hosts, $sortOrder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$pagingLine = CPagerHelper::paginate($page_num, $hosts, $sortOrder, new CUrl('hosts.php'));
 
 	$hosts = API::Host()->get([
 		'output' => API_OUTPUT_EXTEND,
@@ -1451,13 +1623,12 @@ else {
 	}
 
 	$data = [
-		'pageFilter' => $pageFilter,
 		'hosts' => $hosts,
 		'paging' => $pagingLine,
+		'page' => $page_num,
 		'filter' => $filter,
 		'sortField' => $sortField,
 		'sortOrder' => $sortOrder,
-		'groupId' => $pageFilter->groupid,
 		'config' => $config,
 		'templates' => $templates,
 		'maintenances' => $db_maintenances,
@@ -1472,7 +1643,6 @@ else {
 	$hostView = new CView('configuration.host.list', $data);
 }
 
-$hostView->render();
-$hostView->show();
+echo $hostView->getOutput();
 
 require_once dirname(__FILE__).'/include/page_footer.php';
