@@ -35,156 +35,50 @@ function DBconnect(&$error) {
 		return false;
 	}
 
-	$result = true;
-
 	$DB['DB'] = null; // global db handler
 	$DB['TRANSACTIONS'] = 0; // level of a nested transaction
 	$DB['TRANSACTION_NO_FAILED_SQLS'] = true; // true - if no statements failed in transaction, false - there are failed statements
 	$DB['SELECT_COUNT'] = 0; // stats
 	$DB['EXECUTE_COUNT'] = 0;
+	$DB['DB'] = null;
 
 	if (!isset($DB['TYPE'])) {
 		$error = 'Unknown database type.';
-		$result = false;
-	}
-	else {
-		switch ($DB['TYPE']) {
-			case ZBX_DB_MYSQL:
-				$DB['DB'] = @mysqli_connect($DB['SERVER'], $DB['USER'], $DB['PASSWORD'], $DB['DATABASE'], $DB['PORT']);
-				if (!$DB['DB']) {
-					$error = 'Error connecting to database: '.trim(mysqli_connect_error());
-					$result = false;
-				}
-				elseif (mysqli_autocommit($DB['DB'], true) === false) {
-					$error = 'Error setting auto commit.';
-					$result = false;
-				}
-				else {
-					DBexecute('SET NAMES utf8');
-				}
-
-				if ($result) {
-					$dbBackend = new MysqlDbBackend();
-				}
-				break;
-			case ZBX_DB_POSTGRESQL:
-				$pg_connection_string =
-					(!empty($DB['SERVER']) ? 'host=\''.pg_connect_escape($DB['SERVER']).'\' ' : '').
-					'dbname=\''.pg_connect_escape($DB['DATABASE']).'\' '.
-					(!empty($DB['USER']) ? 'user=\''.pg_connect_escape($DB['USER']).'\' ' : '').
-					(!empty($DB['PASSWORD']) ? 'password=\''.pg_connect_escape($DB['PASSWORD']).'\' ' : '').
-					(!empty($DB['PORT']) ? 'port='.pg_connect_escape($DB['PORT']) : '');
-
-				$DB['DB']= @pg_connect($pg_connection_string);
-				if (!$DB['DB']) {
-					$error = 'Error connecting to database.';
-					$result = false;
-				}
-				else {
-					$schemaSet = DBexecute('SET search_path = '.zbx_dbstr($DB['SCHEMA'] ? $DB['SCHEMA'] : 'public'), true);
-
-					if(!$schemaSet) {
-						clear_messages();
-						$error = pg_last_error();
-						$result = false;
-					}
-					else {
-						if (false !== ($pgsql_version = pg_parameter_status('server_version'))) {
-							if ((int) $pgsql_version >= 9) {
-								// change the output format for values of type bytea from hex (the default) to escape
-								DBexecute('SET bytea_output = escape');
-							}
-						}
-					}
-				}
-
-				if ($result) {
-					$dbBackend = new PostgresqlDbBackend();
-				}
-				break;
-			case ZBX_DB_ORACLE:
-				$connect = '';
-				if (!empty($DB['SERVER'])) {
-					$connect = '//'.$DB['SERVER'];
-
-					if ($DB['PORT'] != '0') {
-						$connect .= ':'.$DB['PORT'];
-					}
-					if ($DB['DATABASE']) {
-						$connect .= '/'.$DB['DATABASE'];
-					}
-				}
-
-				$DB['DB'] = @oci_connect($DB['USER'], $DB['PASSWORD'], $connect, 'UTF8');
-				if ($DB['DB']) {
-					DBexecute('ALTER SESSION SET NLS_NUMERIC_CHARACTERS='.zbx_dbstr('. '));
-				}
-				else {
-					$ociError = oci_error();
-					$error = 'Error connecting to database: '.$ociError['message'];
-					$result = false;
-				}
-
-				if ($result) {
-					$dbBackend = new OracleDbBackend();
-				}
-				break;
-			case ZBX_DB_DB2:
-				$connect = '';
-				$connect .= 'DATABASE='.$DB['DATABASE'].';';
-				$connect .= 'HOSTNAME='.$DB['SERVER'].';';
-				$connect .= 'PORT='.$DB['PORT'].';';
-				$connect .= 'PROTOCOL=TCPIP;';
-				$connect .= 'UID='.$DB['USER'].';';
-				$connect .= 'PWD='.$DB['PASSWORD'].';';
-
-				$unicodeprefixes = ['C', 'en_US', 'en_GB'];
-				foreach ($unicodeprefixes as $prefix) {
-					$result = setlocale(LC_ALL, [$prefix.'.utf8', $prefix.'.UTF-8']);
-					if ($result) {
-						break;
-					}
-				}
-				if ($result) {
-					$DB['DB'] = @db2_connect($connect, $DB['USER'], $DB['PASSWORD']);
-					if (!$DB['DB']) {
-						$error = 'Error connecting to database: '.db2_conn_errormsg();
-						$result = false;
-					}
-					else {
-						$options = [
-							'db2_attr_case' => DB2_CASE_LOWER
-						];
-						db2_set_option($DB['DB'], $options, 1);
-						if (isset($DB['SCHEMA']) && $DB['SCHEMA'] != '') {
-							DBexecute('SET CURRENT SCHEMA='.zbx_dbstr($DB['SCHEMA']));
-						}
-					}
-				}
-				else {
-					$error = 'Cannot set UTF-8 locale for web server.';
-				}
-
-				if ($result) {
-					$dbBackend = new Db2DbBackend();
-				}
-				break;
-			default:
-				$error = 'Unsupported database';
-				$result = false;
-		}
+		return false;
 	}
 
-	if ($result && (!$dbBackend->checkDbVersion() || !$dbBackend->checkConfig())) {
-		$error = $dbBackend->getError();
-		$result = false;
+	$db_types = [
+		ZBX_DB_MYSQL => MysqlDbBackend::class,
+		ZBX_DB_POSTGRESQL => PostgresqlDbBackend::class,
+		ZBX_DB_ORACLE => OracleDbBackend::class
+	];
+
+	if (!array_key_exists($DB['TYPE'], $db_types)) {
+		$error = 'Unsupported database';
+		return false;
 	}
 
-	if (false == $result) {
-		$DB['DB'] = null;
+	$db = new $db_types[$DB['TYPE']];
+
+	if ($DB['ENCRYPTION']) {
+		$db->setConnectionSecurity($DB['KEY_FILE'], $DB['CERT_FILE'], $DB['CA_FILE'], $DB['VERIFY_HOST'],
+			$DB['CIPHER_LIST']
+		);
 	}
 
-	return $result;
+	$DB['DB'] = $db->connect($DB['SERVER'], $DB['PORT'], $DB['USER'], $DB['PASSWORD'], $DB['DATABASE'], $DB['SCHEMA']);
+
+	if ($DB['DB']) {
+		$db->init();
+	}
+
+	if ($db->getError() || ($DB['ENCRYPTION'] && !$db->isConnectionSecure()) || !$db->checkDbVersion()
+			|| !$db->checkConfig()) {
+		$error = $db->getError();
+		return false;
+	}
+
+	return true;
 }
 
 function DBclose() {
@@ -203,9 +97,6 @@ function DBclose() {
 			case ZBX_DB_ORACLE:
 				$result = oci_close($DB['DB']);
 				break;
-			case ZBX_DB_DB2:
-				$result = db2_close($DB['DB']);
-				break;
 		}
 	}
 	unset($DB['DB']);
@@ -239,9 +130,6 @@ function DBstart() {
 		case ZBX_DB_ORACLE:
 			$result = true;
 			break;
-		case ZBX_DB_DB2:
-			$result = db2_autocommit($DB['DB'], DB2_AUTOCOMMIT_OFF);
-			break;
 	}
 	return $result;
 }
@@ -296,12 +184,6 @@ function DBcommit() {
 		case ZBX_DB_ORACLE:
 			$result = oci_commit($DB['DB']);
 			break;
-		case ZBX_DB_DB2:
-			$result = db2_commit($DB['DB']);
-			if ($result) {
-				db2_autocommit($DB['DB'], DB2_AUTOCOMMIT_ON);
-			}
-			break;
 	}
 	return $result;
 }
@@ -321,10 +203,6 @@ function DBrollback() {
 		case ZBX_DB_ORACLE:
 			$result = oci_rollback($DB['DB']);
 			break;
-		case ZBX_DB_DB2:
-			$result = db2_rollback($DB['DB']);
-			db2_autocommit($DB['DB'], DB2_AUTOCOMMIT_ON);
-			break;
 	}
 	return $result;
 }
@@ -380,22 +258,6 @@ function DBselect($query, $limit = null, $offset = 0) {
 				error('SQL error ['.$e['message'].'] in ['.$e['sqltext'].']', 'sql');
 			}
 			break;
-		case ZBX_DB_DB2:
-			$options = [];
-			if ($DB['TRANSACTIONS']) {
-				$options['autocommit'] = DB2_AUTOCOMMIT_OFF;
-			}
-
-			if (!$result = db2_prepare($DB['DB'], $query)) {
-				$e = @db2_stmt_errormsg($result);
-				error('SQL error ['.$query.'] in ['.$e.']', 'sql');
-			}
-			elseif (true !== @db2_execute($result, $options)) {
-				$e = @db2_stmt_errormsg($result);
-				error('SQL error ['.$query.'] in ['.$e.']', 'sql');
-				$result = false;
-			}
-			break;
 	}
 
 	// $result is false only if an error occurred
@@ -426,7 +288,7 @@ function DBselect($query, $limit = null, $offset = 0) {
  * PostgreSQL:
  * SELECT a FROM tbl LIMIT 10 OFFSET 5
  *
- * Oracle, DB2:
+ * Oracle:
  * SELECT a FROM tbl WHERE rownum < 15 // ONLY < 15
  * SELECT * FROM (SELECT * FROM tbl) WHERE rownum BETWEEN 6 AND 15
  *
@@ -455,7 +317,6 @@ function DBaddLimit($query, $limit = 0, $offset = 0) {
 				$query .= $offset != 0 ? ' OFFSET '.intval($offset) : '';
 				break;
 			case ZBX_DB_ORACLE:
-			case ZBX_DB_DB2:
 				$till = $offset + $limit;
 				$query = 'SELECT * FROM ('.$query.') WHERE rownum BETWEEN '.intval($offset).' AND '.intval($till);
 				break;
@@ -465,7 +326,7 @@ function DBaddLimit($query, $limit = 0, $offset = 0) {
 	return $query;
 }
 
-function DBexecute($query, $skip_error_messages = 0) {
+function DBexecute($query) {
 	global $DB;
 
 	if (!isset($DB['DB']) || empty($DB['DB'])) {
@@ -501,19 +362,6 @@ function DBexecute($query, $skip_error_messages = 0) {
 				$result = true; // function must return boolean
 			}
 			break;
-		case ZBX_DB_DB2:
-			if (!$result = db2_prepare($DB['DB'], $query)) {
-				$e = @db2_stmt_errormsg($result);
-				error('SQL error ['.$query.'] in ['.$e.']', 'sql');
-			}
-			elseif (true !== @db2_execute($result)) {
-				$e = @db2_stmt_errormsg($result);
-				error('SQL error ['.$query.'] in ['.$e.']', 'sql');
-			}
-			else {
-				$result = true; // function must return boolean
-			}
-			break;
 	}
 	if ($DB['TRANSACTIONS'] != 0 && !$result) {
 		$DB['TRANSACTION_NO_FAILED_SQLS'] = false;
@@ -572,21 +420,6 @@ function DBfetch($cursor, $convertNulls = true) {
 				}
 			}
 			break;
-		case ZBX_DB_DB2:
-			if (!$result = db2_fetch_assoc($cursor)) {
-				db2_free_result($cursor);
-			}
-			else {
-				// cast all of the values to string to be consistent with other DB drivers: all of them return
-				// only strings.
-				foreach ($result as &$value) {
-					if ($value !== null) {
-						$value = (string) $value;
-					}
-				}
-				unset($value);
-			}
-			break;
 	}
 
 	if ($result) {
@@ -676,6 +509,7 @@ function zbx_db_distinct($sql_parts) {
 }
 
 function zbx_db_search($table, $options, &$sql_parts) {
+	global $DB;
 	list($table, $tableShort) = explode(' ', $table);
 
 	$tableSchema = DB::getSchema($table);
@@ -684,56 +518,59 @@ function zbx_db_search($table, $options, &$sql_parts) {
 	}
 
 	$start = $options['startSearch'] ? '' : '%';
-	$exclude = $options['excludeSearch'] ? ' NOT ' : '';
-	$glue = (!$options['searchByAny']) ? ' AND ' : ' OR ';
+	$exclude = $options['excludeSearch'] ? ' NOT' : '';
+	$glue = $options['searchByAny'] ? ' OR ' : ' AND ';
 
 	$search = [];
 	foreach ($options['search'] as $field => $patterns) {
-		if (!isset($tableSchema['fields'][$field]) || zbx_empty($patterns)) {
+		if (!isset($tableSchema['fields'][$field]) || $patterns === null) {
 			continue;
 		}
-		if ($tableSchema['fields'][$field]['type'] != DB::FIELD_TYPE_CHAR
-			&& $tableSchema['fields'][$field]['type'] != DB::FIELD_TYPE_TEXT) {
+
+		$patterns = array_filter((array)$patterns, function($pattern) {
+			return ($pattern !== '');
+		});
+
+		if (!$patterns) {
+			continue;
+		}
+
+		if ($tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_CHAR
+				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_NCLOB
+				&& $tableSchema['fields'][$field]['type'] !== DB::FIELD_TYPE_TEXT) {
 			continue;
 		}
 
 		$fieldSearch = [];
-		foreach ((array) $patterns as $pattern) {
-			if (zbx_empty($pattern)) {
-				continue;
-			}
-
+		foreach ($patterns as $pattern) {
 			// escaping parameter that is about to be used in LIKE statement
-			$pattern = str_replace("!", "!!", $pattern);
-			$pattern = str_replace("%", "!%", $pattern);
-			$pattern = str_replace("_", "!_", $pattern);
+			$pattern = mb_strtoupper(strtr($pattern, ['!' => '!!', '%' => '!%', '_' => '!_']));
 
-			if (!$options['searchWildcardsEnabled']) {
-				$fieldSearch[] =
-					' UPPER('.$tableShort.'.'.$field.') '.
-					$exclude.' LIKE '.
-					zbx_dbstr($start.mb_strtoupper($pattern).'%').
-					" ESCAPE '!'";
+			$pattern = !$options['searchWildcardsEnabled']
+				? $start.$pattern.'%'
+				: str_replace('*', '%', $pattern);
+
+			if ($DB['TYPE'] == ZBX_DB_ORACLE && $tableSchema['fields'][$field]['type'] === DB::FIELD_TYPE_NCLOB
+					&& strlen($pattern) > ORACLE_MAX_STRING_SIZE) {
+				$chunks = zbx_dbstr(DB::chunkMultibyteStr($pattern, ORACLE_MAX_STRING_SIZE));
+				$pattern = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
 			}
 			else {
-				$pattern = str_replace("*", "%", $pattern);
-				$fieldSearch[] =
-					' UPPER('.$tableShort.'.'.$field.') '.
-					$exclude.' LIKE '.
-					zbx_dbstr(mb_strtoupper($pattern)).
-					" ESCAPE '!'";
+				$pattern = zbx_dbstr($pattern);
 			}
+
+			$fieldSearch[] = 'UPPER('.$tableShort.'.'.$field.')'.$exclude.' LIKE '.$pattern." ESCAPE '!'";
 		}
 
-		$search[$field] = '( '.implode($glue, $fieldSearch).' )';
+		$search[$field] = '('.implode($glue, $fieldSearch).')';
 	}
 
-	if (!empty($search)) {
+	if ($search) {
 		if (isset($sql_parts['where']['search'])) {
 			$search[] = $sql_parts['where']['search'];
 		}
 
-		$sql_parts['where']['search'] = '( '.implode($glue, $search).' )';
+		$sql_parts['where']['search'] = '('.implode($glue, $search).')';
 		return true;
 	}
 
@@ -1037,7 +874,7 @@ function pg_connect_escape($string) {
 
 /**
  * Escape string for safe usage in SQL queries.
- * Works for ibmdb2, mysql, oracle, postgresql.
+ * Works for mysql, oracle, postgresql.
  *
  * @param array|string $var
  *
@@ -1051,15 +888,6 @@ function zbx_dbstr($var) {
 	}
 
 	switch ($DB['TYPE']) {
-		case ZBX_DB_DB2:
-			if (is_array($var)) {
-				foreach ($var as $vnum => $value) {
-					$var[$vnum] = "'".db2_escape_string($value)."'";
-				}
-				return $var;
-			}
-			return "'".db2_escape_string($var)."'";
-
 		case ZBX_DB_MYSQL:
 			if (is_array($var)) {
 				foreach ($var as $vnum => $value) {
@@ -1094,7 +922,7 @@ function zbx_dbstr($var) {
 
 /**
  * Creates db dependent string with sql expression that casts passed value to bigint.
- * Works for ibmdb2, mysql, oracle, postgresql.
+ * Works for mysql, oracle, postgresql.
  *
  * @param int $field
  *
@@ -1108,7 +936,6 @@ function zbx_dbcast_2bigint($field) {
 	}
 
 	switch ($DB['TYPE']) {
-		case ZBX_DB_DB2:
 		case ZBX_DB_POSTGRESQL:
 			return 'CAST('.$field.' AS BIGINT)';
 
