@@ -18,10 +18,10 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-define('ZABBIX_VERSION',		'4.4.8');
-define('ZABBIX_API_VERSION',	'4.4.8');
-define('ZABBIX_EXPORT_VERSION',	'4.4');
-define('ZABBIX_DB_VERSION',		4040000);
+define('ZABBIX_VERSION',		'5.0.2');
+define('ZABBIX_API_VERSION',	'5.0.2');
+define('ZABBIX_EXPORT_VERSION',	'5.0');
+define('ZABBIX_DB_VERSION',		5000000);
 
 define('ZABBIX_COPYRIGHT_FROM',	'2001');
 define('ZABBIX_COPYRIGHT_TO',	'2020');
@@ -29,11 +29,15 @@ define('ZABBIX_COPYRIGHT_TO',	'2020');
 define('ZBX_LOGIN_ATTEMPTS',	5);
 define('ZBX_LOGIN_BLOCK',		30); // sec
 
+define('ZBX_BCRYPT_COST',		10);
+define('ZBX_MD5_SIZE',			32);
+
 define('ZBX_SESSION_NAME', 'zbx_sessionid'); // Session cookie name for Zabbix front-end.
 
 define('ZBX_KIBIBYTE',	'1024');
 define('ZBX_MEBIBYTE',	'1048576');
 define('ZBX_GIBIBYTE',	'1073741824');
+define('ZBX_TEBIBYTE',	'1099511627776');
 
 define('ZBX_MIN_PERIOD',		60); // 1 minute
 // The maximum period for the time bar control, ~2 years (2 * 365 * 86400) + 86400 + 3600 + 1.
@@ -41,15 +45,23 @@ define('ZBX_MIN_PERIOD',		60); // 1 minute
 define('ZBX_MAX_PERIOD',		63162001);
 define('ZBX_MIN_INT32',			-2147483648);
 define('ZBX_MAX_INT32',			2147483647);
-define('ZBX_MIN_INT64',			'-9223372036854775808');
-define('ZBX_MAX_INT64',			'9223372036854775807');
 define('ZBX_MAX_UINT64',		'18446744073709551615');
-define('ZBX_MAX_DATE',			2147483647); // 19 Jan 2038 05:14:07
+
+// Double precision 64-bit float.
+define('ZBX_FLOAT_DIG', PHP_FLOAT_DIG);
+define('ZBX_FLOAT_MIN', PHP_FLOAT_MIN);
+define('ZBX_FLOAT_MAX', PHP_FLOAT_MAX);
+
+define('ZBX_MAX_DATE',			ZBX_MAX_INT32); // 19 Jan 2038 05:14:07
 define('ZBX_PERIOD_DEFAULT_FROM',	'now-1h'); // Default time interval.
 define('ZBX_PERIOD_DEFAULT_TO',		'now');
 define('ZBX_MIN_TIMESHIFT',	-788400000); // Min valid timeshift value in seconds (25 years).
 define('ZBX_MAX_TIMESHIFT',	788400000); // Max valid timeshift value in seconds (25 years).
 
+// Applied to tables with variable count of columns. Columns above this limit will be truncated.
+define('ZBX_MAX_TABLE_COLUMNS', 50);
+define('ZBX_MAX_GRAPHS_PER_PAGE', 20);
+
 // Date and time format separators must be synced with setSDateFromOuterObj() in class.calendar.js.
 define('ZBX_FULL_DATE_TIME',	'Y-m-d H:i:s'); // Time selector full date and time presentation format.
 define('ZBX_DATE_TIME',			'Y-m-d H:i'); // Time selector date and time without seconds presentation format.
@@ -73,12 +85,8 @@ define('GRAPH_YAXIS_SIDE_DEFAULT', 0); // 0 - LEFT SIDE, 1 - RIGHT SIDE
 
 define('ZBX_MAX_IMAGE_SIZE', ZBX_MEBIBYTE);
 
-define('ZBX_UNITS_ROUNDOFF_THRESHOLD',		0.01);
-define('ZBX_UNITS_ROUNDOFF_UPPER_LIMIT',	2);
-define('ZBX_UNITS_ROUNDOFF_MIDDLE_LIMIT',	4);
-define('ZBX_UNITS_ROUNDOFF_LOWER_LIMIT',	6);
-
-define('ZBX_PRECISION_10',	10);
+define('ZBX_UNITS_ROUNDOFF_SUFFIXED',		2);
+define('ZBX_UNITS_ROUNDOFF_UNSUFFIXED',		4);
 
 define('ZBX_DEFAULT_INTERVAL', '1-7,00:00-24:00');
 
@@ -88,6 +96,9 @@ define('ZBX_SCRIPT_TYPE_SSH',			2);
 define('ZBX_SCRIPT_TYPE_TELNET',		3);
 define('ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT',	4);
 
+define('ZBX_SEARCH_TYPE_STRICT',	0);
+define('ZBX_SEARCH_TYPE_PATTERN',	1);
+
 define('ZBX_SCRIPT_EXECUTE_ON_AGENT',	0);
 define('ZBX_SCRIPT_EXECUTE_ON_SERVER',	1);
 define('ZBX_SCRIPT_EXECUTE_ON_PROXY',	2);
@@ -116,16 +127,19 @@ define('ZBX_AUTH_HTTP_DISABLED',	0);
 define('ZBX_AUTH_HTTP_ENABLED',		1);
 define('ZBX_AUTH_LDAP_DISABLED',	0);
 define('ZBX_AUTH_LDAP_ENABLED',		1);
+define('ZBX_AUTH_SAML_DISABLED',	0);
+define('ZBX_AUTH_SAML_ENABLED',		1);
 define('ZBX_AUTH_FORM_ZABBIX',	0);
 define('ZBX_AUTH_FORM_HTTP',	1);
 define('ZBX_AUTH_CASE_INSENSITIVE',	0);
 define('ZBX_AUTH_CASE_SENSITIVE',	1);
 
-define('ZBX_DB_DB2',		'IBM_DB2');
 define('ZBX_DB_MYSQL',		'MYSQL');
 define('ZBX_DB_ORACLE',		'ORACLE');
 define('ZBX_DB_POSTGRESQL',	'POSTGRESQL');
 
+define('ZBX_DB_EXTENSION_TIMESCALEDB', 'timescaledb');
+
 define('ZBX_DB_MAX_ID', '9223372036854775807');
 
 // maximum number of records for create() or update() API calls
@@ -134,6 +148,7 @@ define('ZBX_DB_MAX_INSERTS', 10000);
 // Default db and field character set
 define('ZBX_DB_DEFAULT_CHARSET', 'UTF8');
 define('ZBX_DB_MYSQL_DEFAULT_COLLATION', 'utf8_bin');
+define('ORACLE_MAX_STRING_SIZE', 4000);
 
 define('ZBX_SHOW_TECHNICAL_ERRORS', false);
 
@@ -150,17 +165,10 @@ define('PAGE_TYPE_TEXT_RETURN_JSON',	11); // input plaintext output json
 define('ZBX_SESSION_ACTIVE',	0);
 define('ZBX_SESSION_PASSIVE',	1);
 
-define('ZBX_DROPDOWN_FIRST_NONE',	0);
-define('ZBX_DROPDOWN_FIRST_ALL',	1);
-
 define('T_ZBX_STR',			0);
 define('T_ZBX_INT',			1);
 define('T_ZBX_DBL',			2);
 define('T_ZBX_RANGE_TIME',	3);
-define('T_ZBX_CLR',			5);
-define('T_ZBX_DBL_BIG',		9);
-define('T_ZBX_DBL_STR',		10);
-define('T_ZBX_TP',			11);
 define('T_ZBX_TU',			12);
 define('T_ZBX_ABS_TIME',	13);
 
@@ -212,6 +220,7 @@ define('AUDIT_ACTION_LOGIN',	3);
 define('AUDIT_ACTION_LOGOUT',	4);
 define('AUDIT_ACTION_ENABLE',	5);
 define('AUDIT_ACTION_DISABLE',	6);
+define('AUDIT_ACTION_EXECUTE',	7);
 
 define('AUDIT_RESOURCE_USER',				0);
 define('AUDIT_RESOURCE_ZABBIX_CONFIG',		2);
@@ -247,6 +256,7 @@ define('AUDIT_RESOURCE_GRAPH_PROTOTYPE',	35);
 define('AUDIT_RESOURCE_ITEM_PROTOTYPE',		36);
 define('AUDIT_RESOURCE_HOST_PROTOTYPE',		37);
 define('AUDIT_RESOURCE_AUTOREGISTRATION',	38);
+define('AUDIT_RESOURCE_MODULE',				39);
 
 define('CONDITION_TYPE_HOST_GROUP',			0);
 define('CONDITION_TYPE_HOST',				1);
@@ -314,6 +324,9 @@ define('HOST_STATUS_TEMPLATE',		3);
 define('HOST_STATUS_PROXY_ACTIVE',	5);
 define('HOST_STATUS_PROXY_PASSIVE',	6);
 
+define('HOST_DISCOVER',		0);
+define('HOST_NO_DISCOVER',	1);
+
 define('HOST_ENCRYPTION_NONE',			1);
 define('HOST_ENCRYPTION_PSK',			2);
 define('HOST_ENCRYPTION_CERTIFICATE',	4);
@@ -345,10 +358,19 @@ define('MAINTENANCE_STATUS_ACTIVE',		0);
 define('MAINTENANCE_STATUS_APPROACH',	1);
 define('MAINTENANCE_STATUS_EXPIRED',	2);
 
+// Modules.
+define('MODULE_STATUS_DISABLED', 0);
+define('MODULE_STATUS_ENABLED',	1);
+
 define('HOST_AVAILABLE_UNKNOWN',	0);
 define('HOST_AVAILABLE_TRUE',		1);
 define('HOST_AVAILABLE_FALSE',		2);
 
+// Logo.
+define('LOGO_TYPE_NORMAL',			0);
+define('LOGO_TYPE_SIDEBAR',			1);
+define('LOGO_TYPE_SIDEBAR_COMPACT',	2);
+
 define('MAINTENANCE_TAG_EVAL_TYPE_AND_OR',	0);
 define('MAINTENANCE_TAG_EVAL_TYPE_OR',		2);
 define('MAINTENANCE_TAG_OPERATOR_EQUAL',	0);
@@ -450,12 +472,12 @@ define('ZBX_ITEM_FLEXIBLE_DELAY_DEFAULT',	'50s');
 define('ZBX_ITEM_SCHEDULING_DEFAULT',		'wd1-5h9-18');
 
 define('ITEM_TYPE_ZABBIX',			0);
-define('ITEM_TYPE_SNMPV1',			1);
+define('ITEM_TYPE_SNMPV1',			1); // Deprecated. Now only used in XML converters. Use ITEM_TYPE_SNMP instead.
 define('ITEM_TYPE_TRAPPER',			2);
 define('ITEM_TYPE_SIMPLE',			3);
-define('ITEM_TYPE_SNMPV2C',			4);
+define('ITEM_TYPE_SNMPV2C',			4); // Deprecated. Now only used in XML converters. Use ITEM_TYPE_SNMP instead.
 define('ITEM_TYPE_INTERNAL',		5);
-define('ITEM_TYPE_SNMPV3',			6);
+define('ITEM_TYPE_SNMPV3',			6); // Deprecated. Now only used in XML converters. Use ITEM_TYPE_SNMP instead.
 define('ITEM_TYPE_ZABBIX_ACTIVE',	7);
 define('ITEM_TYPE_AGGREGATE',		8);
 define('ITEM_TYPE_HTTPTEST',		9);
@@ -469,6 +491,11 @@ define('ITEM_TYPE_JMX',				16);
 define('ITEM_TYPE_SNMPTRAP',		17);
 define('ITEM_TYPE_DEPENDENT',		18);
 define('ITEM_TYPE_HTTPAGENT',		19);
+define('ITEM_TYPE_SNMP',			20);
+
+define('SNMP_V1', 1);
+define('SNMP_V2C', 2);
+define('SNMP_V3', 3);
 
 define('ZBX_DEPENDENT_ITEM_MAX_LEVELS',	3);
 define('ZBX_DEPENDENT_ITEM_MAX_COUNT',	29999);
@@ -484,8 +511,8 @@ define('ITEM_DATA_TYPE_OCTAL',			1);
 define('ITEM_DATA_TYPE_HEXADECIMAL',	2);
 define('ITEM_DATA_TYPE_BOOLEAN',		3);
 
-define('ZBX_DEFAULT_KEY_DB_MONITOR',			'db.odbc.select[<unique short description>,dsn]');
-define('ZBX_DEFAULT_KEY_DB_MONITOR_DISCOVERY',	'db.odbc.discovery[<unique short description>,dsn]');
+define('ZBX_DEFAULT_KEY_DB_MONITOR',			'db.odbc.select[<unique short description>,<dsn>,<connection string>]');
+define('ZBX_DEFAULT_KEY_DB_MONITOR_DISCOVERY',	'db.odbc.discovery[<unique short description>,<dsn>,<connection string>]');
 define('ZBX_DEFAULT_KEY_SSH',					'ssh.run[<unique short description>,<ip>,<port>,<encoding>]');
 define('ZBX_DEFAULT_KEY_TELNET',				'telnet.run[<unique short description>,<ip>,<port>,<encoding>]');
 
@@ -499,6 +526,8 @@ define('ZBX_ICON_PREVIEW_WIDTH',	24);
 
 define('ITEM_STATUS_ACTIVE',		0);
 define('ITEM_STATUS_DISABLED',		1);
+define('ITEM_DISCOVER',	0);
+define('ITEM_NO_DISCOVER',	1);
 
 /**
  * Starting from Zabbix 2.2 items could not have ITEM_STATUS_NOTSUPPORTED status
@@ -558,6 +587,7 @@ define('ZBX_PREPROC_SCRIPT',				21);
 define('ZBX_PREPROC_PROMETHEUS_PATTERN',	22);
 define('ZBX_PREPROC_PROMETHEUS_TO_JSON',	23);
 define('ZBX_PREPROC_CSV_TO_JSON',			24);
+define('ZBX_PREPROC_STR_REPLACE',			25);
 
 // Item pre-processing error handlers.
 define('ZBX_PREPROC_FAIL_DEFAULT',			0);
@@ -568,6 +598,21 @@ define('ZBX_PREPROC_FAIL_SET_ERROR',		3);
 define('ZBX_PREPROC_CSV_NO_HEADER',	0);
 define('ZBX_PREPROC_CSV_HEADER',	1);
 
+// LLD rule overrides.
+define('ZBX_LLD_OVERRIDE_STOP_NO',	0);
+define('ZBX_LLD_OVERRIDE_STOP_YES',	1);
+define('ZBX_PROTOTYPE_STATUS_ENABLED', 0);
+define('ZBX_PROTOTYPE_STATUS_DISABLED', 1);
+define('ZBX_PROTOTYPE_DISCOVER', 0);
+define('ZBX_PROTOTYPE_NO_DISCOVER', 1);
+define('OPERATION_OBJECT_ITEM_PROTOTYPE', 0);
+define('OPERATION_OBJECT_TRIGGER_PROTOTYPE', 1);
+define('OPERATION_OBJECT_GRAPH_PROTOTYPE', 2);
+define('OPERATION_OBJECT_HOST_PROTOTYPE', 3);
+
+define('GRAPH_DISCOVER',	0);
+define('GRAPH_NO_DISCOVER',	1);
+
 define('GRAPH_ITEM_DRAWTYPE_LINE',			0);
 define('GRAPH_ITEM_DRAWTYPE_FILLED_REGION',	1);
 define('GRAPH_ITEM_DRAWTYPE_BOLD_LINE',		2);
@@ -607,6 +652,8 @@ define('ZBX_RECOVERY_MODE_NONE',				2);
 
 define('TRIGGER_STATUS_ENABLED',	0);
 define('TRIGGER_STATUS_DISABLED',	1);
+define('TRIGGER_DISCOVER',		0);
+define('TRIGGER_NO_DISCOVER',	1);
 
 define('TRIGGER_VALUE_FALSE',	0);
 define('TRIGGER_VALUE_TRUE',	1);
@@ -658,30 +705,6 @@ define('SMTP_AUTHENTICATION_NORMAL',	1);
 define('SMTP_MESSAGE_FORMAT_PLAIN_TEXT',	0);
 define('SMTP_MESSAGE_FORMAT_HTML',			1);
 
-define('ACTION_DEFAULT_SUBJ_AUTOREG', 'Auto registration: {HOST.HOST}');
-define('ACTION_DEFAULT_SUBJ_DISCOVERY', 'Discovery: {DISCOVERY.DEVICE.STATUS} {DISCOVERY.DEVICE.IPADDRESS}');
-define('ACTION_DEFAULT_SUBJ_ACKNOWLEDGE', 'Updated problem: {EVENT.NAME}');
-define('ACTION_DEFAULT_SUBJ_PROBLEM', 'Problem: {EVENT.NAME}');
-define('ACTION_DEFAULT_SUBJ_RECOVERY', 'Resolved: {EVENT.NAME}');
-
-define('ACTION_DEFAULT_MSG_AUTOREG', "Host name: {HOST.HOST}\nHost IP: {HOST.IP}\nAgent port: {HOST.PORT}");
-define('ACTION_DEFAULT_MSG_DISCOVERY', "Discovery rule: {DISCOVERY.RULE.NAME}\n\n".
-		"Device IP: {DISCOVERY.DEVICE.IPADDRESS}\nDevice DNS: {DISCOVERY.DEVICE.DNS}\n".
-		"Device status: {DISCOVERY.DEVICE.STATUS}\nDevice uptime: {DISCOVERY.DEVICE.UPTIME}\n\n".
-		"Device service name: {DISCOVERY.SERVICE.NAME}\nDevice service port: {DISCOVERY.SERVICE.PORT}\n".
-		"Device service status: {DISCOVERY.SERVICE.STATUS}\nDevice service uptime: {DISCOVERY.SERVICE.UPTIME}"
-);
-define('ACTION_DEFAULT_MSG_ACKNOWLEDGE',
-		"{USER.FULLNAME} {EVENT.UPDATE.ACTION} problem at {EVENT.UPDATE.DATE} {EVENT.UPDATE.TIME}.\n".
-		"{EVENT.UPDATE.MESSAGE}\n\n".
-		"Current problem status is {EVENT.STATUS}, acknowledged: {EVENT.ACK.STATUS}."
-);
-define('ACTION_DEFAULT_MSG_PROBLEM', "Problem started at {EVENT.TIME} on {EVENT.DATE}\nProblem name: {EVENT.NAME}\n".
-		"Host: {HOST.NAME}\nSeverity: {EVENT.SEVERITY}\n\nOriginal problem ID: {EVENT.ID}\n{TRIGGER.URL}");
-define('ACTION_DEFAULT_MSG_RECOVERY', "Problem has been resolved at {EVENT.RECOVERY.TIME} on {EVENT.RECOVERY.DATE}\n".
-		"Problem name: {EVENT.NAME}\nHost: {HOST.NAME}\nSeverity: {EVENT.SEVERITY}\n\n".
-		"Original problem ID: {EVENT.ID}\n{TRIGGER.URL}");
-
 define('ACTION_STATUS_ENABLED',		0);
 define('ACTION_STATUS_DISABLED',	1);
 
@@ -731,7 +754,6 @@ define('SCREEN_RESOURCE_SYSTEM_STATUS',		15);
 define('SCREEN_RESOURCE_HOST_TRIGGERS',		16);
 // used in Monitoring > Latest data > Graph (history.php)
 define('SCREEN_RESOURCE_HISTORY',			17);
-define('SCREEN_RESOURCE_CHART',				18);
 define('SCREEN_RESOURCE_LLD_SIMPLE_GRAPH',	19);
 define('SCREEN_RESOURCE_LLD_GRAPH',			20);
 // used in Monitoring > Web > Details (httpdetails.php)
@@ -940,6 +962,7 @@ define('ZBX_PROBLEM_UPDATE_CLOSE',			0x01);
 define('ZBX_PROBLEM_UPDATE_ACKNOWLEDGE',	0x02);
 define('ZBX_PROBLEM_UPDATE_MESSAGE',		0x04);
 define('ZBX_PROBLEM_UPDATE_SEVERITY',		0x08);
+define('ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE',	0x10);
 
 define('ZBX_EVENT_HISTORY_PROBLEM_EVENT',		0);
 define('ZBX_EVENT_HISTORY_RECOVERY_EVENT',		1);
@@ -953,10 +976,10 @@ define('ZBX_TM_TASK_CHECK_NOW',		6);
 define('ZBX_TM_STATUS_NEW',			1);
 define('ZBX_TM_STATUS_INPROGRESS',	2);
 
-define('EVENT_SOURCE_TRIGGERS',				0);
-define('EVENT_SOURCE_DISCOVERY',			1);
-define('EVENT_SOURCE_AUTO_REGISTRATION',	2);
-define('EVENT_SOURCE_INTERNAL', 			3);
+define('EVENT_SOURCE_TRIGGERS',			0);
+define('EVENT_SOURCE_DISCOVERY',		1);
+define('EVENT_SOURCE_AUTOREGISTRATION',	2);
+define('EVENT_SOURCE_INTERNAL',			3);
 
 define('EVENT_OBJECT_TRIGGER',			0);
 define('EVENT_OBJECT_DHOST',			1);
@@ -1138,18 +1161,43 @@ define('UNKNOWN_VALUE', '');
 define('ZBX_EOL_LF',	0);
 define('ZBX_EOL_CRLF',	1);
 
-// suffixes
-define('ZBX_BYTE_SUFFIXES', 'KMGT');
+// Time intervals.
+define('SEC_PER_MIN',			60);
+define('SEC_PER_HOUR',			3600);
+define('SEC_PER_DAY',			86400);
+define('SEC_PER_WEEK',			604800);
+define('SEC_PER_MONTH',			2592000);
+define('SEC_PER_YEAR',			31536000);
+
+// Time suffixes and multipliers.
 define('ZBX_TIME_SUFFIXES', 'smhdw');
 define('ZBX_TIME_SUFFIXES_WITH_YEAR', 'smhdwMy');
+define('ZBX_TIME_SUFFIX_MULTIPLIERS', [
+	's' => 1,
+	'm' => SEC_PER_MIN,
+	'h' => SEC_PER_HOUR,
+	'd' => SEC_PER_DAY,
+	'w' => SEC_PER_WEEK,
+	'M' => SEC_PER_MONTH,
+	'y' => SEC_PER_YEAR
+]);
 
-// preg
+// Byte suffixes and multipliers.
+define('ZBX_BYTE_SUFFIXES', 'KMGT');
+define('ZBX_BYTE_SUFFIX_MULTIPLIERS', [
+	'K' => ZBX_KIBIBYTE,
+	'M' => ZBX_MEBIBYTE,
+	'G' => ZBX_GIBIBYTE,
+	'T' => ZBX_TEBIBYTE
+]);
+
+// Regular expressions.
 define('ZBX_PREG_PRINT', '^\x00-\x1F');
 define('ZBX_PREG_MACRO_NAME', '([A-Z0-9\._]+)');
 define('ZBX_PREG_MACRO_NAME_LLD', '([A-Z0-9\._]+)');
 define('ZBX_PREG_INTERNAL_NAMES', '([0-9a-zA-Z_\. \-]+)'); // !!! Don't forget sync code with C !!!
-define('ZBX_PREG_NUMBER', '([\-+]?[0-9]+[.]?[0-9]*['.ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES.']?)');
-define('ZBX_PREG_INT', '([\-+]?[0-9]+['.ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES.']?)');
+define('ZBX_PREG_NUMBER', '(?<number>-?(\d+(\.\d*)?|\.\d+)([Ee][+-]?\d+)?)');
+define('ZBX_PREG_INT', '(?<int>-?\d+)');
 define('ZBX_PREG_DEF_FONT_STRING', '/^[0-9\.:% ]+$/');
 define('ZBX_PREG_DNS_FORMAT', '([0-9a-zA-Z_\.\-$]|\{\$?'.ZBX_PREG_MACRO_NAME.'\})*');
 define('ZBX_PREG_HOST_FORMAT', ZBX_PREG_INTERNAL_NAMES);
@@ -1282,6 +1330,7 @@ define('API_NUMERIC',			28);
 define('API_LLD_MACRO',			29);
 define('API_PSK',				30);
 define('API_SORTORDER',			31);
+define('API_CALC_FORMULA',		32);
 
 // flags
 define('API_REQUIRED',					0x0001);
@@ -1295,6 +1344,7 @@ define('API_ALLOW_LLD_MACRO',			0x0080);
 define('API_REQUIRED_LLD_MACRO',		0x0100);
 define('API_TIME_UNIT_WITH_YEAR',		0x0200);
 define('API_ALLOW_EVENT_TAGS_MACRO',	0x0400);
+define('API_PRESERVE_KEYS',				0x0800);
 
 // JSON error codes.
 if (!defined('JSON_ERROR_NONE')) {
@@ -1314,13 +1364,6 @@ define('ZBX_API_ERROR_NO_METHOD',	300);
 define('API_OUTPUT_EXTEND',		'extend');
 define('API_OUTPUT_COUNT',		'count');
 
-define('SEC_PER_MIN',			60);
-define('SEC_PER_HOUR',			3600);
-define('SEC_PER_DAY',			86400);
-define('SEC_PER_WEEK',			604800);
-define('SEC_PER_MONTH',			2592000);
-define('SEC_PER_YEAR',			31536000);
-
 define('ZBX_JAN_2038', 2145916800);
 
 define('DAY_IN_YEAR', 365);
@@ -1328,17 +1371,26 @@ define('DAY_IN_YEAR', 365);
 define('ZBX_MIN_PORT_NUMBER', 0);
 define('ZBX_MAX_PORT_NUMBER', 65535);
 
+define('ZBX_MACRO_TYPE_TEXT', 0); // Display macro value as text.
+define('ZBX_MACRO_TYPE_SECRET', 1); // Display masked macro value.
+define('ZBX_MACRO_SECRET_MASK', '******'); // Placeholder for masked macro value.
+
 // Layout
 define('ZBX_LAYOUT_NORMAL',     0);
-define('ZBX_LAYOUT_FULLSCREEN', 1);
-define('ZBX_LAYOUT_KIOSKMODE',  2);
+define('ZBX_LAYOUT_KIOSKMODE',  1);
 define('ZBX_LAYOUT_MODE', 'layout-mode');
 
+// Sidebar
+define('ZBX_SIDEBAR_VIEW_MODE_FULL',	0);
+define('ZBX_SIDEBAR_VIEW_MODE_COMPACT',	1);
+define('ZBX_SIDEBAR_VIEW_MODE_HIDDEN',	2);
+
 // input fields
 define('ZBX_TEXTAREA_HTTP_PAIR_NAME_WIDTH',		218);
 define('ZBX_TEXTAREA_HTTP_PAIR_VALUE_WIDTH',	218);
 define('ZBX_TEXTAREA_MACRO_WIDTH',				250);
 define('ZBX_TEXTAREA_MACRO_VALUE_WIDTH',		300);
+define('ZBX_TEXTAREA_MACRO_INHERITED_WIDTH',	180);
 define('ZBX_TEXTAREA_TAG_WIDTH',				250);
 define('ZBX_TEXTAREA_TAG_VALUE_WIDTH',			300);
 define('ZBX_TEXTAREA_COLOR_WIDTH',				96);
@@ -1368,6 +1420,7 @@ define('ZBX_OVERVIEW_HELP_MIN_WIDTH',			125);
 define('ZBX_ACTION_ADD',		0);
 define('ZBX_ACTION_REPLACE',	1);
 define('ZBX_ACTION_REMOVE',		2);
+define('ZBX_ACTION_REMOVE_ALL', 3);
 
 // Maximum width for popups in Actions column for problems.
 define('ZBX_ACTIONS_POPUP_MAX_WIDTH',			800);
@@ -1501,6 +1554,11 @@ define('ITEM_NO_STORAGE_VALUE',	0);
 // configuration -> maps default add icon name
 define('MAP_DEFAULT_ICON', 'Server_(96)');
 
+// Condition popup types.
+define('ZBX_POPUP_CONDITION_TYPE_EVENT_CORR', 0);
+define('ZBX_POPUP_CONDITION_TYPE_ACTION', 1);
+define('ZBX_POPUP_CONDITION_TYPE_ACTION_OPERATION', 2);
+
 // CSS styles
 define('ZBX_STYLE_ACTION_BUTTONS', 'action-buttons');
 define('ZBX_STYLE_ADM_IMG', 'adm-img');
@@ -1525,23 +1583,20 @@ define('ZBX_STYLE_BTN_GREY', 'btn-grey');
 define('ZBX_STYLE_BTN_INFO', 'btn-info');
 define('ZBX_STYLE_BTN_LINK', 'btn-link');
 define('ZBX_STYLE_BTN_KIOSK', 'btn-kiosk');
-define('ZBX_STYLE_BTN_MAX', 'btn-max');
 define('ZBX_STYLE_BTN_MIN', 'btn-min');
 define('ZBX_STYLE_BTN_REMOVE_FAV', 'btn-remove-fav');
-define('ZBX_STYLE_BTN_SEARCH', 'btn-search');
 define('ZBX_STYLE_BTN_TIME', 'btn-time');
 define('ZBX_STYLE_BTN_TIME_LEFT', 'btn-time-left');
 define('ZBX_STYLE_BTN_TIME_OUT', 'btn-time-out');
 define('ZBX_STYLE_BTN_TIME_RIGHT', 'btn-time-right');
 define('ZBX_STYLE_BTN_WIDGET_ACTION', 'btn-widget-action');
 define('ZBX_STYLE_BTN_WIDGET_COLLAPSE', 'btn-widget-collapse');
-define('ZBX_STYLE_BTN_WIDGET_DELETE', 'btn-widget-delete');
 define('ZBX_STYLE_BTN_WIDGET_EDIT', 'btn-widget-edit');
 define('ZBX_STYLE_BTN_WIDGET_EXPAND', 'btn-widget-expand');
 define('ZBX_STYLE_BOTTOM', 'bottom');
 define('ZBX_STYLE_BROWSER_LOGO_CHROME', 'browser-logo-chrome');
 define('ZBX_STYLE_BROWSER_LOGO_FF', 'browser-logo-ff');
-define('ZBX_STYLE_BROWSER_LOGO_IE', 'browser-logo-ie');
+define('ZBX_STYLE_BROWSER_LOGO_ED', 'browser-logo-ed');
 define('ZBX_STYLE_BROWSER_LOGO_OPERA', 'browser-logo-opera');
 define('ZBX_STYLE_BROWSER_LOGO_SAFARI', 'browser-logo-safari');
 define('ZBX_STYLE_BROWSER_WARNING_CONTAINER', 'browser-warning-container');
@@ -1577,6 +1632,7 @@ define('ZBX_STYLE_DASHED_BORDER', 'dashed-border');
 define('ZBX_STYLE_DEBUG_OUTPUT', 'debug-output');
 define('ZBX_STYLE_DISABLED', 'disabled');
 define('ZBX_STYLE_DISASTER_BG', 'disaster-bg');
+define('ZBX_STYLE_DISPLAY_NONE', 'display-none');
 define('ZBX_STYLE_DRAG_ICON', 'drag-icon');
 define('ZBX_STYLE_PROBLEM_UNACK_FG', 'problem-unack-fg');
 define('ZBX_STYLE_PROBLEM_ACK_FG', 'problem-ack-fg');
@@ -1607,14 +1663,15 @@ define('ZBX_STYLE_FLH_NA_BG', 'flh-na-bg');
 define('ZBX_STYLE_FLH_WARNING_BG', 'flh-warning-bg');
 define('ZBX_STYLE_FLOAT_LEFT', 'float-left');
 define('ZBX_STYLE_FORM_INPUT_MARGIN', 'form-input-margin');
+define('ZBX_STYLE_FORM_FIELDS_INLINE', 'form-fields-inline');
 define('ZBX_STYLE_FORM_NEW_GROUP', 'form-new-group');
 define('ZBX_STYLE_GRAPH_WRAPPER', 'graph-wrapper');
 define('ZBX_STYLE_GREEN', 'green');
 define('ZBX_STYLE_GREEN_BG', 'green-bg');
 define('ZBX_STYLE_GREY', 'grey');
 define('ZBX_STYLE_TEAL', 'teal');
-define('ZBX_STYLE_HEADER_LOGO', 'header-logo');
 define('ZBX_STYLE_HEADER_TITLE', 'header-title');
+define('ZBX_STYLE_HEADER_CONTROLS', 'header-controls');
 define('ZBX_STYLE_HIGH_BG', 'high-bg');
 define('ZBX_STYLE_HOR_LIST', 'hor-list');
 define('ZBX_STYLE_HOVER_NOBG', 'hover-nobg');
@@ -1627,7 +1684,6 @@ define('ZBX_STYLE_ICON_INFO', 'icon-info');
 define('ZBX_STYLE_ICON_INVISIBLE', 'icon-invisible');
 define('ZBX_STYLE_ICON_MAINT', 'icon-maint');
 define('ZBX_STYLE_ICON_WZRD_ACTION', 'icon-wzrd-action');
-define('ZBX_STYLE_ICON_NONE', 'icon-none');
 define('ZBX_STYLE_ACTION_COMMAND', 'icon-action-command');
 define('ZBX_STYLE_ACTION_ICON_CLOSE', 'icon-action-close');
 define('ZBX_STYLE_ACTION_ICON_MSG', 'icon-action-msg');
@@ -1637,6 +1693,7 @@ define('ZBX_STYLE_ACTION_ICON_SEV_DOWN', 'icon-action-severity-down');
 define('ZBX_STYLE_ACTION_ICON_SEV_CHANGED', 'icon-action-severity-changed');
 define('ZBX_STYLE_ACTION_MESSAGE', 'icon-action-message');
 define('ZBX_STYLE_ACTION_ICON_ACK', 'icon-action-ack');
+define('ZBX_STYLE_ACTION_ICON_UNACK', 'icon-action-unack');
 define('ZBX_STYLE_PROBLEM_GENERATED', 'icon-problem-generated');
 define('ZBX_STYLE_PROBLEM_RECOVERY', 'icon-problem-recovery');
 define('ZBX_STYLE_ACTIONS_NUM_GRAY', 'icon-actions-number-gray');
@@ -1646,6 +1703,7 @@ define('ZBX_STYLE_INACTIVE_BG', 'inactive-bg');
 define('ZBX_STYLE_INFO_BG', 'info-bg');
 define('ZBX_STYLE_INPUT_COLOR_PICKER', 'input-color-picker');
 define('ZBX_STYLE_LAYOUT_KIOSKMODE', 'layout-kioskmode');
+define('ZBX_STYLE_LAYOUT_WRAPPER', 'wrapper');
 define('ZBX_STYLE_LEFT', 'left');
 define('ZBX_STYLE_LINK_ACTION', 'link-action');
 define('ZBX_STYLE_LINK_ALT', 'link-alt');
@@ -1691,8 +1749,8 @@ define('ZBX_STYLE_OVERFLOW_ELLIPSIS', 'overflow-ellipsis');
 define('ZBX_STYLE_OBJECT_GROUP', 'object-group');
 define('ZBX_STYLE_PAGING_BTN_CONTAINER', 'paging-btn-container');
 define('ZBX_STYLE_PAGING_SELECTED', 'paging-selected');
-define('ZBX_STYLE_PRELOADER', 'preloader');
 define('ZBX_STYLE_PAGE_TITLE', 'page-title-general');
+define('ZBX_STYLE_PAGE_TITLE_SUBMENU', 'page-title-submenu');
 define('ZBX_STYLE_PROGRESS_BAR_BG', 'progress-bar-bg');
 define('ZBX_STYLE_PROGRESS_BAR_CONTAINER', 'progress-bar-container');
 define('ZBX_STYLE_PROGRESS_BAR_LABEL', 'progress-bar-label');
@@ -1705,6 +1763,7 @@ define('ZBX_STYLE_ROW', 'row');
 define('ZBX_STYLE_INLINE_SR_ONLY', 'inline-sr-only');
 define('ZBX_STYLE_SCREEN_TABLE', 'screen-table');
 define('ZBX_STYLE_SEARCH', 'search');
+define('ZBX_STYLE_FORM_SEARCH', 'form-search');
 define('ZBX_STYLE_SECOND_COLUMN_LABEL', 'second-column-label');
 define('ZBX_STYLE_SELECTED', 'selected');
 define('ZBX_STYLE_SELECTED_ITEM_COUNT', 'selected-item-count');
@@ -1743,6 +1802,7 @@ define('ZBX_STYLE_TABLE_FORMS_CONTAINER', 'table-forms-container');
 define('ZBX_STYLE_TABLE_FORMS_SECOND_COLUMN', 'table-forms-second-column');
 define('ZBX_STYLE_TABLE_FORMS_TD_LEFT', 'table-forms-td-left');
 define('ZBX_STYLE_TABLE_FORMS_TD_RIGHT', 'table-forms-td-right');
+define('ZBX_STYLE_TABLE_FORMS_OVERFLOW_BREAK', 'overflow-break');
 define('ZBX_STYLE_TABLE_PAGING', 'table-paging');
 define('ZBX_STYLE_TABLE_STATS', 'table-stats');
 define('ZBX_STYLE_TABS_NAV', 'tabs-nav');
@@ -1760,17 +1820,6 @@ define('ZBX_STYLE_TIMELINE_DOT_BIG', 'timeline-dot-big');
 define('ZBX_STYLE_TIMELINE_TD', 'timeline-td');
 define('ZBX_STYLE_TIMELINE_TH', 'timeline-th');
 define('ZBX_STYLE_TOP', 'top');
-define('ZBX_STYLE_TOP_NAV', 'top-nav');
-define('ZBX_STYLE_TOP_NAV_CONTAINER', 'top-nav-container');
-define('ZBX_STYLE_TOP_NAV_HELP', 'top-nav-help');
-define('ZBX_STYLE_TOP_NAV_ICONS', 'top-nav-icons');
-define('ZBX_STYLE_TOP_NAV_PROFILE', 'top-nav-profile');
-define('ZBX_STYLE_TOP_NAV_GUEST', 'top-nav-guest');
-define('ZBX_STYLE_TOP_NAV_SIGNOUT', 'top-nav-signout');
-define('ZBX_STYLE_TOP_NAV_SUPPORT', 'top-nav-support');
-define('ZBX_STYLE_TOP_NAV_ZBBSHARE', 'top-nav-zbbshare');
-define('ZBX_STYLE_TOP_SUBNAV', 'top-subnav');
-define('ZBX_STYLE_TOP_SUBNAV_CONTAINER', 'top-subnav-container');
 define('ZBX_STYLE_TOTALS_LIST', 'totals-list');
 define('ZBX_STYLE_TOTALS_LIST_HORIZONTAL', 'totals-list-horizontal');
 define('ZBX_STYLE_TOTALS_LIST_VERTICAL', 'totals-list-vertical');
@@ -1783,9 +1832,15 @@ define('ZBX_STYLE_WIDGET_URL', 'widget-url');
 define('ZBX_STYLE_BLINK_HIDDEN', 'blink-hidden');
 define('ZBX_STYLE_YELLOW', 'yellow');
 define('ZBX_STYLE_FIELD_LABEL_ASTERISK', 'form-label-asterisk');
+define('ZBX_STYLE_PROBLEM_ICON_LIST' , 'problem-icon-list');
+define('ZBX_STYLE_PROBLEM_ICON_LIST_ITEM' , 'problem-icon-list-item');
+define('ZBX_STYLE_ZABBIX_LOGO', 'zabbix-logo');
+define('ZBX_STYLE_ZABBIX_SIDEBAR_LOGO', 'zabbix-sidebar-logo');
+define('ZBX_STYLE_ZABBIX_SIDEBAR_LOGO_COMPACT', 'zabbix-sidebar-logo-compact');
 
 // HTML column layout.
 define('ZBX_STYLE_COLUMNS', 'columns-wrapper');
+define('ZBX_STYLE_COLUMNS_NOWRAP', 'columns-nowrap');
 define('ZBX_STYLE_COLUMNS_2', 'columns-2');
 define('ZBX_STYLE_COLUMNS_3', 'columns-3');
 // column occupies x% width of column wrapper
@@ -1815,6 +1870,32 @@ define('ZBX_STYLE_HOST_AVAIL_TOTAL', 'host-avail-total');
 // Widget "Problems by severity" styles.
 define('ZBX_STYLE_BY_SEVERITY_WIDGET', 'by-severity-widget');
 
+define('ZBX_STYLE_CHECKBOX_BLOCK', 'checkbox-block');
+
+// Icons.
+define('ZBX_STYLE_ICON_TEXT', 'icon-text');
+define('ZBX_STYLE_ICON_SECRET_TEXT', 'icon-secret');
+
+// Host interface styles.
+define('ZBX_STYLE_HOST_INTERFACE_CONTAINER', 'interface-container');
+define('ZBX_STYLE_HOST_INTERFACE_CONTAINER_HEADER', 'interface-container-header');
+define('ZBX_STYLE_HOST_INTERFACE_ROW', 'interface-row');
+define('ZBX_STYLE_HOST_INTERFACE_ROW_HEADER', 'interface-row-header');
+define('ZBX_STYLE_HOST_INTERFACE_CELL', 'interface-cell');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_DETAILS', 'interface-cell-details');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_HEADER', 'interface-cell-header');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_TYPE', 'interface-cell-type');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_IP', 'interface-cell-ip');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_DNS', 'interface-cell-dns');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_USEIP', 'interface-cell-useip');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_PORT', 'interface-cell-port');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_DEFAULT', 'interface-cell-default');
+define('ZBX_STYLE_HOST_INTERFACE_CELL_ACTION', 'interface-cell-action');
+define('ZBX_STYLE_HOST_INTERFACE_BTN_TOGGLE', 'interface-btn-toggle');
+define('ZBX_STYLE_HOST_INTERFACE_BTN_REMOVE', 'interface-btn-remove');
+define('ZBX_STYLE_HOST_INTERFACE_BTN_MAIN_INTERFACE', 'interface-btn-main-interface');
+define('ZBX_STYLE_HOST_INTERFACE_INPUT_EXPAND', 'interface-input-expand');
+
 // server variables
 define('HTTPS', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] && $_SERVER['HTTPS'] !== 'off');
 
@@ -1849,6 +1930,14 @@ define('OPERATIONAL_DATA_SHOW_NONE',         0);
 define('OPERATIONAL_DATA_SHOW_SEPARATELY',   1);
 define('OPERATIONAL_DATA_SHOW_WITH_PROBLEM', 2);
 
+/**
+ * The sandbox attribute enables an extra set of restrictions for the content in the iframe. Default is set to empty
+ * string, which means all restrictions are applied. To disable, set to FALSE. To set a specific set of restrictions,
+ * write a custom string.
+ * https://www.w3.org/TR/2010/WD-html5-20100624/the-iframe-element.html#attr-iframe-sandbox
+ */
+define('ZBX_IFRAME_SANDBOX', '');
+
 // HTTP headers
 /*
  * Value of HTTP X-Frame-options header.
