@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,12 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
+$this->includeJsFile('monitoring.screen.js.php');
+
 $widget = (new CWidget())->setTitle(_('Screens').': '.$data['screen']['name']);
 
 if (array_key_exists('templateid', $data['screen'])) {
@@ -38,4 +44,4 @@ $widget->addItem(
 $screenBuilder->insertInitScreenJs($data['screenid']);
 $screenBuilder->insertProcessObjectsJs();
 
-return $widget;
+$widget->show();
